/*******************************************************************************
* FILE NAME: LCD_Samples.c
*
* DESCRIPTION:
*  Sample uses of a 4-line 20 character/line LCD
*  The LCD used with these examples was:
*		Model: BPP-420L
*		4x20 Serial LCD Module v4.0
*		Scott Edwards Electronics, Inc.
*		www.seetron.com
*  The BPP-420L is RS232/Inverted-TTL compatible, has a 64-byte buffer, and a large text mode
*  
*******************************************************************************/

#include <string.h>
#include "serial_comm.h"
#include "LCD_Samples.h"
#include "write_utilities.h"

/*******************************************************************************
* FUNCTION NAME: LCD_Samples
* PURPOSE:       Displays a series of repeating LCD screen manipulation samples
*******************************************************************************/
void LCD_Samples()
{
  char status;
  static unsigned char flag=0, fleas=95; // This will overflow and wrap around to repeat
  static char init=0;

	if (0 == flag)
	{
		init=0; //For later use
	}

	if (0 == flag)  // Write BIG 358 only once
	{
    	Write_Large_Sample(); //Demonstrate the LCD's Large Character mode
	}
//--------------------------
	else if (60 == flag) // Let it be seen for awhile
	{
  		Write_Form_Sample_2(0);  //Update particular positions on the LCD
	}
	else if (80 == flag)  //Do this a few times
	{
  		Write_Form_Sample_2(3);  //Update particular positions on the LCD
	}
	else if (100 == flag)  //Do this a few times
	{
  		Write_Form_Sample_2(5);  //Update particular positions on the LCD
	}
	else if (120 == flag)  //Do this a few times
	{
  		Write_Form_Sample_2(8);  //Update particular positions on the LCD
	}
//--------------------------
	else if (150 == flag)  //Let's write something else
	{
  		Write_Byte(CLEAR);
  		WriteString((char *) "My Dog Has ");
		WriteSignedDecimal(fleas++); //Display a number, any number
  		WriteString((char *) " Fleas");
	}
//--------------------------
	else if (180 < flag)  //Histograms
	{
		if (0 == flag%5)  //Don't update faster than we can write or the LCD can update (3 was too fast)
		{
			Write_Form_Sample_3(init);
			if (0==init) //Initialize the display only once
			{
				init = 1;
			}
		}
	}
    flag++;
}


/*******************************************************************************
* FUNCTION NAME: Write_Form_Sample_1
* PURPOSE:       Displays a simple text string
*
* RETURN:
*   0 if successful, 1 if unsuccessful (queue is full)
*******************************************************************************/
rom const char string1[] = "My Cat Has Fleas";

char Write_Form_Sample_1()
{
    char status, i, error=SERIAL_SUCCESSFUL;

    //Simple text string
    Write_Byte(CLEAR);
    for (i=0; i<sizeof(string1) ;i++)
    {
        status = Write_Byte(string1[i]);
        if (status != SERIAL_SUCCESSFUL)
        {
            error == SERIAL_UNSUCCESSFUL;
        }
    }
    return(error); // Results may be indeterminate if failure
}


/*******************************************************************************
* FUNCTION NAME: Write_Form_Sample_2
* PURPOSE:       Displays and updates only particular LCD positions
*
* RETURN:
*   0 if successful, 1 if unsuccessful (queue is full)
*******************************************************************************/
#define FORM1_A 45
#define FORM1_B 75

char Write_Form_Sample_2(char choice)
{
    char status, error=SERIAL_SUCCESSFUL;
 
    //Update particular LCD locations that are changing
			if (0 == choice) //Initialize LCD
			{ // Display the fixed text of the form only once
					status = Write_Byte(CLEAR);
					status = Write_Byte(POSITION);
					status = Write_Byte(64+FORM1_A-22);
					status = Write_Byte(80); // "P"
					status = Write_Byte(111); // "o"
					status = Write_Byte(115); // "s"
					status = Write_Byte(65); // "A"
					status = Write_Byte(45); // "-"

					status = Write_Byte(POSITION);
					status = Write_Byte(64+FORM1_B-22);
					status = Write_Byte(80); // "P"
					status = Write_Byte(111); // "o"
					status = Write_Byte(115); // "s"
					status = Write_Byte(45); // "-"
					status = Write_Byte(66); // "B"
			}
			else
			{ // Display only changes to the form
					status = Write_Byte(POSITION);
					status = Write_Byte(64+FORM1_A);
					//Convert # to text
					WriteSignedDecimal(choice);
					status = Write_Byte(POSITION);
					status = Write_Byte(64+FORM1_B);
					WriteSignedDecimal(choice);
			}
    return(error); // Results may be indeterminate if failure
}


/*******************************************************************************
* FUNCTION NAME: Write_Form_Sample_3
* PURPOSE:       Displays an active histogram on the LCD 
*                This might serve as a tach screen, for example
*				 Do this efficiently so it doesn't take way too long to send
*				 Only send changes to the previous display
* RETURN:
*   0 if successful, 1 if unsuccessful (queue is full)
*******************************************************************************/
#define APOS 4
#define BPOS 44

char Write_Form_Sample_3(char init)
{
    char i, status, error=SERIAL_SUCCESSFUL;
	static char H1=8, H2=8, Adirection=1, Bdirection=-1;

    //Sample histogram
	if (0 == init) //Initialize LCD
	{ // Display the fixed text of the form only once
		Write_Byte(CLEAR);
  		Write_Byte(65); //"A" 
  		Write_Byte(CR); //Carriage Return
  		Write_Byte(CR); //Carriage Return
  		Write_Byte(66); //"B"
		status = Write_Byte(POSITION);
		status = Write_Byte(64+APOS);
    	for (i=0; i<H1; i++) status = Write_Byte(BLOCK); // Block character
		status = Write_Byte(POSITION);
		status = Write_Byte(64+BPOS);
   		for (i=0; i<H2; i++) status = Write_Byte(BLOCK); // Block character
    }
	else
	{
		status = Write_Byte(POSITION);
		status = Write_Byte(64+APOS+H1);
		if (Adirection > 0)
		{
   	 		status = Write_Byte(BLOCK); // Add Block
		}
		else
		{
    		status = Write_Byte(BLANK); // Erase Block
		}
		H1 += Adirection;
		if ((14<H1) || (1>H1))
		{
			Adirection *= -1;
		}

		status = Write_Byte(POSITION);
		status = Write_Byte(64+BPOS+H2);
		if (Bdirection > 0)
		{
	    	status = Write_Byte(BLOCK); // Add Block
		}
		else
		{
	    	status = Write_Byte(BLANK); // Erase Block
		}
		H2 += Bdirection;
		if ((14<H2) || (1>H2))
		{
			Bdirection *= -1;
		}
	}
    return(error); // Results may be indeterminate if failure
}

/*******************************************************************************
* FUNCTION NAME: Write_Large_Sample
* PURPOSE:      Demonstrate the large character mode of an LCD
*				A 4-line, 20-character/line LCD will show 4 big characters.
*				At 2400 BAUD we take only 4ms to send a character (too fast)
*				A simple solution is to send only one byte per slow loop
*		 	Each large character takes 10ms for the LCD to display, so give it time
*******************************************************************************/

void Write_Large_Sample()
{
    // Write BIG 358
  	Write_Byte(CLEAR);
  	Write_Byte(START_BIG);
	Write_Byte(51);  //"3"
	Write_Byte(53);  //"5"
	Write_Byte(56);  //"8"
	Write_Byte(END_BIG);
//	Write_Byte(BACKLIGHT_ON); // Light it up for no particular reason
}
